"""MkDocs documentation file management.

Provides MkDocs documentation pages: docs/index.md (home page with badges) and
docs/api.md (API reference with mkdocstrings).

Modules:
    index: docs/index.md configuration management
    api: docs/api.md configuration management

See Also:
    pyrig.dev.configs.docs.mkdocs.MkdocsConfigFile
"""
